package ru.amse.smartlang.pascal.format;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import ru.amse.smartlang.format.IBlock;
import ru.amse.smartlang.format.IRuleSet;
import ru.amse.smartlang.format.RuleSet;
import ru.amse.smartlang.format.RuleSetConstructor;
import ru.amse.smartlang.format.WhitespaceConverter;
import ru.amse.smartlang.format.formatter.BlockTreePrinter;
import ru.amse.smartlang.format.formatter.Formatter;
import ru.amse.smartlang.pascal.format.parser.MyParser;
import ru.amse.smartlang.pascal.format.parser.PascalLexer;
import antlr.RecognitionException;
import antlr.TokenStreamException;

public class Main {
	
	private static IBlock parse(String s) throws RecognitionException, TokenStreamException, FileNotFoundException {
		IBlock res = new MyParser(new PascalLexer(new FileReader(s))).program_block();
		WhitespaceConverter.toRelativeWhitespaces(res);
		return res;
	}


	public static void main(String[] args) {
		if(args.length == 0) {
			System.out.println("Pascal code formatter utility.\nUsage format: java -jar pasformat.jar <samplefile.pas> <filetoformat.pas>");
			return;
		}
		if(args.length != 2) {
			System.out.println("Invalid format. Run without parametrs to get short usage description.");
			
		}
		IBlock sample;
		try {
			sample = parse(args[0]);
		} catch (RecognitionException e) {
			System.out.println("Error parsing sample file : " + e.getMessage());
			return;
		} catch (TokenStreamException e) {
			System.out.println("Error parsing sample file : " + e.getMessage());
			return;
		} catch (FileNotFoundException e) {
			System.out.println("Sample file not found : " + e.getMessage());
			return;
		}
		IBlock fmt;
		try {
			fmt = parse(args[1]);
		} catch (RecognitionException e) {
			System.out.println("Error parsing input file : " + e.getMessage());
			return;
		} catch (TokenStreamException e) {
			System.out.println("Error parsing input file : " + e.getMessage());
			return;
		} catch (FileNotFoundException e) {
			System.out.println("Input file not found : " + e.getMessage());
			return;
		}
		IRuleSet ruleSet = new RuleSet();
		new RuleSetConstructor(ruleSet).construct(sample);
		new Formatter(ruleSet).format(fmt);
		WhitespaceConverter.toAbsoluteWhitespaces(fmt);
		Writer out;
		try {
			out = new FileWriter(args[1]);
		} catch (IOException e) {
			System.err.println("Error opening file : " + e.getMessage());
			return;
		}
		new BlockTreePrinter().print(fmt, out);
		try {
			out.close();
		} catch (IOException e) {
			System.err.println("Error writing to file : " + e.getMessage());
		}
	}
}
